/* Automatically generated by po2tbl.sed from goom2.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"Goom Control Window", 2},
  {"_File", 3},
  {"Window Y-Size:", 4},
  {"Window X-Size:", 5},
  {"Double Pixel:", 6},
  {"Frame Rate Limiter:", 7},
  {"Display", 8},
  {"TODO", 9},
  {"Inspect", 10},
  {"Customize", 11},
  {"Couldn't find pixmap file: %s", 12},
  {"Error loading pixmap file: %s", 13},
};

int _msg_tbl_length = 13;
