#ifndef _GOOM_README_H
#define _GOOM_README_H

/* Automatically generated from README, via Makefile.am */
static char    goom_readme[] =
"TAB - Enter/Leave Fullscreen\n"
"Numpad +/- - Change resolution\n"
"F1-F8 - Set Background FX\n"
"\n"
"F - Display Frame Rate\n"
"Q - Quit\n";

static char    goom_big_readme[] =
"****************************\n"
"* What a GOOM! version 2k4 *\n"
"****************************\n"
"by Jean-Christophe 'Jeko' Hoelt\n"
"\n"
"\n"
"The incredible trippy visualization plugin!!\n"
"\n"
"\n"
"\n"
"...... freely offered by iOS-Software\n"
"Wanna find the last version of goom or support\n"
"us with a donation? There you go:\n"
"\n"
"http://www.ios-software.com/\n"
"\n"
"\n"
"\n"
"...... many thanks to:\n"
"\n"
"Skal for the code of IFS\n"
"\n"
"Gyom for the iTunes version\n"
"         and some of the main work\n"
"\n"
"Fred for the Windows version and a lot of\n"
"         other Goom Stuff\n"
"\n"
"Dennis and guys of libvisual for their\n"
"         help on the unix package\n"
"\n"
"\n"
"\n"
"Enjoy!";

#endif /* _GOOM_README_H */
