/*
 * DO NOT EDIT THIS FILE - it is generated by Glade.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "gtk-callbacks.h"
#include "gtk-interface.h"
#include "gtk-support.h"

GtkWidget*
create_config_window (void)
{
  GtkWidget *config_window;
  GtkWidget *vbox1;
  GtkWidget *toolbar1;
  GtkWidget *b_open_config;
  GtkWidget *b_save_config;
  GtkWidget *notebook1;
  GtkWidget *scrolledwindow1;
  GtkWidget *text1;
  GtkWidget *goom_control;
  GtkWidget *statusbar1;

  config_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_set_name (config_window, "config_window");
  gtk_object_set_data (GTK_OBJECT (config_window), "config_window", config_window);
  gtk_window_set_title (GTK_WINDOW (config_window), _("Goom Control Center"));

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_set_name (vbox1, "vbox1");
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (config_window), vbox1);

  toolbar1 = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_TEXT);
  gtk_widget_set_name (toolbar1, "toolbar1");
  gtk_widget_ref (toolbar1);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "toolbar1", toolbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (toolbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), toolbar1, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (toolbar1), 3);
  gtk_toolbar_set_space_size (GTK_TOOLBAR (toolbar1), 8);
  gtk_toolbar_set_space_style (GTK_TOOLBAR (toolbar1), GTK_TOOLBAR_SPACE_LINE);

  b_open_config = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Open..."),
                                NULL, NULL,
                                NULL, NULL, NULL);
  gtk_widget_set_name (b_open_config, "b_open_config");
  gtk_widget_ref (b_open_config);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "b_open_config", b_open_config,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_open_config);

  b_save_config = gtk_toolbar_append_element (GTK_TOOLBAR (toolbar1),
                                GTK_TOOLBAR_CHILD_BUTTON,
                                NULL,
                                _("Save..."),
                                NULL, NULL,
                                NULL, NULL, NULL);
  gtk_widget_set_name (b_save_config, "b_save_config");
  gtk_widget_ref (b_save_config);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "b_save_config", b_save_config,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (b_save_config);

  notebook1 = gtk_notebook_new ();
  gtk_widget_set_name (notebook1, "notebook1");
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox1), notebook1, TRUE, TRUE, 0);
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (notebook1), GTK_POS_LEFT);

  scrolledwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_set_name (scrolledwindow1, "scrolledwindow1");
  gtk_widget_ref (scrolledwindow1);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "scrolledwindow1", scrolledwindow1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (scrolledwindow1);
  gtk_container_add (GTK_CONTAINER (notebook1), scrolledwindow1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1), GTK_POLICY_NEVER, GTK_POLICY_ALWAYS);

  text1 = gtk_text_new (NULL, NULL);
  gtk_widget_set_name (text1, "text1");
  gtk_widget_ref (text1);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "text1", text1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (text1);
  gtk_container_add (GTK_CONTAINER (scrolledwindow1), text1);
  gtk_widget_set_usize (text1, 400, 300);
  gtk_text_insert (GTK_TEXT (text1), NULL, NULL, NULL,
                   _("What a GOOM!! v2\n\ncopyright 2000-2003, by J.C. Hoelt <jeko@free.fr>\n\nThis is my first visual plugins for XMMS, and I\nthink that it's the best I have ever done !\n\n---\n\nThis dialog will help you to configure goom.\nMany options here will seem strange to you,\nanyway, just try things and look what happen !\n\nEnjoy,\n\n\tJeko"), 317);

  goom_control = gtk_label_new (_("About goom..."));
  gtk_widget_set_name (goom_control, "goom_control");
  gtk_widget_ref (goom_control);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "goom_control", goom_control,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (goom_control);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), goom_control);

  statusbar1 = gtk_statusbar_new ();
  gtk_widget_set_name (statusbar1, "statusbar1");
  gtk_widget_ref (statusbar1);
  gtk_object_set_data_full (GTK_OBJECT (config_window), "statusbar1", statusbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, FALSE, 0);

  gtk_signal_connect (GTK_OBJECT (config_window), "destroy_event",
                      GTK_SIGNAL_FUNC (on_config_window_destroy_event),
                      NULL);
  gtk_signal_connect (GTK_OBJECT (config_window), "delete_event",
                      GTK_SIGNAL_FUNC (on_config_window_delete_event),
                      NULL);

  return config_window;
}

